/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.spark.sql.sedona_sql.expressions

import org.apache.spark.sql.catalyst.expressions.Expression
import org.apache.spark.sql.Column
import org.apache.spark.sql.functions.{col, udaf}

object st_aggregates {
  def ST_Envelope_Aggr(geometry: Column): Column = {
    val aggrFunc = udaf(new ST_Envelope_Aggr)
    aggrFunc(geometry)
  }

  def ST_Envelope_Aggr(geometry: String): Column = {
    val aggrFunc = udaf(new ST_Envelope_Aggr)
    aggrFunc(col(geometry))
  }

  def ST_Intersection_Aggr(geometry: Column): Column = {
    val aggrFunc = udaf(new ST_Intersection_Aggr)
    aggrFunc(geometry)
  }

  def ST_Intersection_Aggr(geometry: String): Column = {
    val aggrFunc = udaf(new ST_Intersection_Aggr)
    aggrFunc(col(geometry))
  }

  def ST_Union_Aggr(geometry: Column): Column = {
    val aggrFunc = udaf(new ST_Union_Aggr)
    aggrFunc(geometry)
  }

  def ST_Union_Aggr(geometry: String): Column = {
    val aggrFunc = udaf(new ST_Union_Aggr)
    aggrFunc(col(geometry))
  }

  def ST_Collect_Agg(geometry: Column): Column = {
    val aggrFunc = udaf(new ST_Collect_Agg)
    aggrFunc(geometry)
  }

  def ST_Collect_Agg(geometry: String): Column = {
    val aggrFunc = udaf(new ST_Collect_Agg)
    aggrFunc(col(geometry))
  }

  // Aliases for *_Aggr functions with *_Agg suffix
  def ST_Envelope_Agg(geometry: Column): Column = ST_Envelope_Aggr(geometry)

  def ST_Envelope_Agg(geometry: String): Column = ST_Envelope_Aggr(geometry)

  def ST_Intersection_Agg(geometry: Column): Column = ST_Intersection_Aggr(geometry)

  def ST_Intersection_Agg(geometry: String): Column = ST_Intersection_Aggr(geometry)

  def ST_Union_Agg(geometry: Column): Column = ST_Union_Aggr(geometry)

  def ST_Union_Agg(geometry: String): Column = ST_Union_Aggr(geometry)
}
